@@ -143,8 +143,7 @@ class SimpleSAML_Module {
 		assert('is_string($resource)');
 		assert('$resource[0] !== "/"');
 
-		$config = SimpleSAML_Configuration::getInstance();
-		$url = SimpleSAML_Utilities::selfURLhost() . '/' . $config->getBaseURL() . 'module.php/' . $resource;
+		$url = SimpleSAML_Utilities::getBaseURL() . 'module.php/' . $resource;
 		if (!empty($parameters)) {
 			$url = SimpleSAML_Utilities::addURLparameter($url, $parameters);
 		}
@@ -163,7 +162,9 @@ class SimpleSAML_Module {
 	public static function callHooks($hook, &$data = NULL) {
 		assert('is_string($hook)');
 
-		foreach (self::getModules() as $module) {
+		$modules = self::getModules();
+		sort($modules);
+		foreach ($modules as $module) {
 			if (!self::isModuleEnabled($module)) {
 				continue;
 			}
