@@ -94,7 +94,7 @@ class sspmod_cas_Auth_Source_CAS  extends SimpleSAML_Auth_Source  {
 				'ticket' => $ticket,
 				'service' => $service,
 		));
-		$result = file_get_contents($url);
+		$result = SimpleSAML_Utilities::fetch($url);
 		$res = preg_split("/\r?\n/",$result);
 
 		if (strcmp($res[0], "yes") == 0) {
@@ -117,7 +117,7 @@ class sspmod_cas_Auth_Source_CAS  extends SimpleSAML_Auth_Source  {
 				'ticket' => $ticket,
 				'service' => $service,
 		));
-		$result = file_get_contents($url);
+		$result = SimpleSAML_Utilities::fetch($url);
 
 		$dom = DOMDocument::loadXML($result);
 		$xPath = new DOMXpath($dom);
