@@ -105,7 +105,7 @@ class sspmod_saml_Auth_Source_SP extends SimpleSAML_Auth_Source {
 		assert('is_string($entityId)');
 
 		if ($this->idp !== NULL && $this->idp !== $entityId) {
-			throw new SimpleSAML_Error_Exception('Cannot retrieve metadata for IdP ' . var_export($this->idp, TRUE) .
+			throw new SimpleSAML_Error_Exception('Cannot retrieve metadata for IdP ' . var_export($entityId, TRUE) .
 				' because it isn\'t a valid IdP for this SP.');
 		}
 
@@ -180,7 +180,7 @@ class sspmod_saml_Auth_Source_SP extends SimpleSAML_Auth_Source {
 			SimpleSAML_Auth_State::throwException($state, new SimpleSAML_Error_ProxyCountExceeded("ProxyCountExceeded"));
 		}
 
-		$ar = sspmod_saml2_Message::buildAuthnRequest($this->metadata, $idpMetadata);
+		$ar = sspmod_saml_Message::buildAuthnRequest($this->metadata, $idpMetadata);
 
 		$ar->setAssertionConsumerServiceURL(SimpleSAML_Module::getModuleURL('saml/sp/saml2-acs.php/' . $this->authId));
 
@@ -202,10 +202,17 @@ class sspmod_saml_Auth_Source_SP extends SimpleSAML_Auth_Source {
 		}
 
 		if (isset($state['saml:NameIDPolicy'])) {
-			$ar->setNameIdPolicy(array(
-				'Format' => (string)$state['saml:NameIDPolicy'],
-				'AllowCreate' => TRUE,
-			));
+			if (is_string($state['saml:NameIDPolicy'])) {
+				$policy = array(
+					'Format' => (string)$state['saml:NameIDPolicy'],
+					'AllowCreate' => TRUE,
+				);
+			} elseif (is_array($state['saml:NameIDPolicy'])) {
+				$policy = $state['saml:NameIDPolicy'];
+			} else {
+				throw new SimpleSAML_Error_Exception('Invalid value of $state[\'saml:NameIDPolicy\'].');
+			}
+			$ar->setNameIdPolicy($policy);
 		}
 
 		if (isset($state['saml:IDPList'])) {
@@ -242,7 +249,6 @@ class sspmod_saml_Auth_Source_SP extends SimpleSAML_Auth_Source {
 
 		SimpleSAML_Logger::debug('Sending SAML 2 AuthnRequest to ' . var_export($idpMetadata->getString('entityid'), TRUE));
 		$b = new SAML2_HTTPRedirect();
-		$b->setDestination(sspmod_SAML2_Message::getDebugDestination());
 		$b->send($ar);
 
 		assert('FALSE');
@@ -366,13 +372,20 @@ class sspmod_saml_Auth_Source_SP extends SimpleSAML_Auth_Source {
 			return;
 		}
 
-		$lr = sspmod_saml2_Message::buildLogoutRequest($this->metadata, $idpMetadata);
+		$lr = sspmod_saml_Message::buildLogoutRequest($this->metadata, $idpMetadata);
 		$lr->setNameId($nameId);
 		$lr->setSessionIndex($sessionIndex);
 		$lr->setRelayState($id);
 
+		$encryptNameId = $idpMetadata->getBoolean('nameid.encryption', NULL);
+		if ($encryptNameId === NULL) {
+			$encryptNameId = $this->metadata->getBoolean('nameid.encryption', FALSE);
+		}
+		if ($encryptNameId) {
+			$lr->encryptNameId(sspmod_saml_Message::getEncryptionKey($idpMetadata));
+		}
+
 		$b = new SAML2_HTTPRedirect();
-		$b->setDestination(sspmod_SAML2_Message::getDebugDestination());
 		$b->send($lr);
 
 		assert('FALSE');
@@ -422,6 +435,8 @@ class sspmod_saml_Auth_Source_SP extends SimpleSAML_Auth_Source {
 
 		$authProcState = array(
 			'saml:sp:IdP' => $idp,
+			'saml:sp:NameID' => $state['saml:sp:NameID'],
+			'saml:sp:SessionIndex' => $state['saml:sp:SessionIndex'],
 			'saml:sp:State' => $state,
 			'ReturnCall' => array('sspmod_saml_Auth_Source_SP', 'onProcessingCompleted'),
 
@@ -473,10 +488,9 @@ class sspmod_saml_Auth_Source_SP extends SimpleSAML_Auth_Source {
 		$source->addLogoutCallback($idp, $state);
 
 		$state['Attributes'] = $authProcState['Attributes'];
-		$state['IdP'] = $idp;
 
 		if (isset($state['saml:sp:isUnsoliced']) && (bool)$state['saml:sp:isUnsoliced']) {
-			if (isset($state['saml:sp:RelayState'])) {
+			if (!empty($state['saml:sp:RelayState'])) {
 				$redirectTo = $state['saml:sp:RelayState'];
 			} else {
 				$redirectTo = $source->getMetadata()->getString('RelayState', '/');
