@@ -138,27 +138,32 @@ if($onLoad !== '') {
 		echo '<div id="languagebar">';
 		$languages = $this->getLanguageList();
 		$langnames = array(
-			'no' => 'Bokmål',
-			'nn' => 'Nynorsk',
-			'se' => 'Sámi',
-			'da' => 'Dansk',
-			'en' => 'English',
-			'de' => 'Deutsch',
-			'sv' => 'Svenska',
-			'fi' => 'Suomeksi',
-			'es' => 'Español',
-			'eu' => 'Euskara',
-			'fr' => 'Français',
-			'nl' => 'Nederlands',
-			'lb' => 'Luxembourgish', 
-			'cs' => 'Czech',
-			'sl' => 'Slovenščina', // Slovensk
-			'hr' => 'Hrvatski', // Croatian
-			'hu' => 'Magyar', // Hungarian
-			'pl' => 'Język polski', // Polish
-			'pt' => 'Português', // Portuguese
-			'pt-BR' => 'Português brasileiro', // Portuguese
-			'tr' => 'Türkçe',
+					'no' => 'Bokmål',
+					'nn' => 'Nynorsk',
+					'se' => 'Sámegiella',
+					'sam' => 'Åarjelh-saemien giele',
+					'da' => 'Dansk',
+					'en' => 'English',
+					'de' => 'Deutsch',
+					'sv' => 'Svenska',
+					'fi' => 'Suomeksi',
+					'es' => 'Español',
+					'fr' => 'Français',
+					'it' => 'Italiano',
+					'nl' => 'Nederlands',
+					'lb' => 'Luxembourgish', 
+					'cs' => 'Czech',
+					'sl' => 'Slovenščina', // Slovensk
+					'lt' => 'Lietuvių kalba', // Lithuanian
+					'hr' => 'Hrvatski', // Croatian
+					'hu' => 'Magyar', // Hungarian
+					'pl' => 'Język polski', // Polish
+					'pt' => 'Português', // Portuguese
+					'pt-BR' => 'Português brasileiro', // Portuguese
+					'tr' => 'Türkçe',
+					'el' => 'ελληνικά',
+					'ja' => '日本語',
+					'zh-tw' => '中文',
 		);
 		
 		$textarray = array();
